/**
 * @file system_lm3s6965.c
 * @brief System configuration for LM3S6965 device
 *
 * @section License
 *
 * Copyright (C) 2010-2018 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneTCP Eval.
 *
 * This software is provided in source form for a short-term evaluation only. The
 * evaluation license expires 90 days after the date you first download the software.
 *
 * If you plan to use this software in a commercial product, you are required to
 * purchase a commercial license from Oryx Embedded SARL.
 *
 * After the 90-day evaluation period, you agree to either purchase a commercial
 * license or delete all copies of this software. If you wish to extend the
 * evaluation period, you must contact sales@oryx-embedded.com.
 *
 * This evaluation software is provided "as is" without warranty of any kind.
 * Technical support is available as an option during the evaluation period.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.9.0
 **/

#include <stdint.h>
#include "lm3s6965.h"
#include "inc/hw_types.h"
#include "driverlib/sysctl.h"
#include "system_lm3s6965.h"

//System clock frequency (50MHz)
uint32_t SystemCoreClock = 50000000;


/**
 * @brief Setup the system clock
 **/

void SystemInit(void)
{
   //Set the clocking to run from the PLL at 50 MHz
   SysCtlClockSet(SYSCTL_OSC_MAIN | SYSCTL_XTAL_8MHZ |
      SYSCTL_USE_PLL | SYSCTL_SYSDIV_4);
}


/**
 * @brief Update the variable SystemCoreClock
 **/

void SystemCoreClockUpdate(void)
{
   //Retrieve system clock
   SystemCoreClock = SysCtlClockGet();
}
